#include "ping.h"

void updateZone(Uint32 ip, Uint16 port, Uint32 population);
void deleteZone(Uint32 ip, Uint16 port);


//////// Ping target ////////

Uint32 pingTarget::getTime()
{
	return GetTickCount() / 10;
}

void pingTarget::sendPing()
{
	Uint32 time = getTime();
	s->set(remote);
	s->send((char*)&time, sizeof(Uint32));

	lastTime = time;
}

pingTarget::pingTarget(UDPSocket * ns, Uint32 ip, Uint16 port)
{
	remote.set(ip, port);
	s = ns;
	pinging = false;
	population = -1;
	highPing = -1;
	lastPing = -1;
}

void pingTarget::startPing()
{
	sendPing();

	pinging = true;
	startTime = getTime();
}

bool pingTarget::doEvents()
{
	if (pinging)
	{
		if (getTime() - startTime > 1000)
		{
			return true;
		}

		if (getTime() - lastTime > highPing)
		{
			sendPing();
		}
	}

	return false;
}

bool pingTarget::validateSource(INADDR * src)
{
	return (src->ip == remote.ip) && (src->port == remote.port);
}

void pingTarget::gotPong(Uint32 time, Uint32 pop)
{
	pinging = false;

	Uint32 t = getTime() - time;

	population = pop;
	lastPing = t;
	if ((highPing == -1) || (t > highPing))
		highPing = t;
}


//////// Ping manager ////////

void pingPort::sendPong(INADDR * src, Uint32 ping)
{
	char msg[8];
	*(Uint32*)&msg[0] = population;
	*(Uint32*)&msg[4] = ping;

	s.set(*src);
	s.send(msg, 8);
}

Uint32 pingPort::getTime()
{
	return GetTickCount() / 10;
}

pingTarget * pingPort::findTarget(INADDR * src)
{
	_listnode <pingTarget> * parse = targets.head;

	while (parse)
	{
		pingTarget * t = parse->item;

		if (t->validateSource(src))
			return t;

		parse = parse->next;
	}

	return NULL;
}

pingPort::pingPort(Uint16 port)
{
	s.create(port);
	population = 0;
	lastPing = 0;
	myTarget = NULL;
}

void pingPort::setPopulation(Uint32 pop)
{
	population = pop;
}

void pingPort::ping(Uint32 ip, Uint16 port)
{
	pingTarget * t = new pingTarget(&s, ip, port);
	if (t)
	{
		targets.append(t);
	}
}

void pingPort::doEvents()
{
	UDPPacket * p = s.poll();

	// Check for ping/pong events
	if (p)
	{
		if (p->len == 8)
		{	// It's a pong
			pingTarget * t = findTarget(&p->src);

			if (t)
			{
				t->gotPong(getLong(p->msg, 4), getLong(p->msg, 0));
				updateZone(t->remote.ip, t->remote.getPort(), getLong(p->msg, 0));
			}
		}
		else if (p->len == 4)
		{	// It's a ping
			sendPong(&p->src, getLong(p->msg, 0));
		}
		else if (p->len == 0x74)
		{
		}


		delete p;
	}

	// Check for internal events
	_listnode <pingTarget> * parse = targets.head;

	while (parse)
	{
		pingTarget * t = parse->item;
		parse = parse->next;

		if (t->doEvents())
		{
			if (myTarget)
			if (myTarget->item == t)
			{
				if (myTarget->next)
					myTarget = myTarget->next;
				else
					myTarget = targets.head;
			}

			deleteZone(t->remote.ip, t->remote.getPort());
			targets.kill(t, false);
		}
	}

	// Sweep list with pings
	if (getTime() - lastPing > SWEEP_TIME)
	{
		if (myTarget == NULL)
			myTarget = targets.head;

		if (myTarget)
		{
			if (myTarget->next)
				myTarget = myTarget->next;
			else
				myTarget = targets.head;

			if (myTarget)
			{
				myTarget->item->startPing();
				lastPing = getTime();
			}
		}
	}
}
